require 'test_helper'

class TestEngine < Minitest::Test
  def setup
    Drawy::Engine.people  = nil
    Drawy::Engine.couples = nil
    Drawy::Engine.result  = nil
  end

  def test_add_people
    x, y, z = ['john', 'anna'], ['dan'], ['camille']

    Drawy::Engine.add_people(x)
    Drawy::Engine.add_people(y)
    Drawy::Engine.add_people(z)

    assert_equal x + y + z, Drawy::Engine.people
  end

  def test_add_people_builds_couples
    x, y = ['john', 'anna'], ['dan', 'camille']

    Drawy::Engine.add_people(x)
    Drawy::Engine.add_people(y)

    # Couples should be of form { 'john' => 'anna', 'dan' => 'camille' }
    assert_equal 'anna',    Drawy::Engine.couples['john']
    assert_equal 'camille', Drawy::Engine.couples['dan']
  end

  def test_add_people_downcases_names
    x = ['John']

    Drawy::Engine.add_people(x)

    assert_equal 'john', Drawy::Engine.people.first
  end

  def test_add_people_deletes_empty_names
    x = ['john', '']

    Drawy::Engine.add_people(x)

    assert_equal ['john'], Drawy::Engine.people
  end

  def test_draw
    Drawy::Engine.people  = %w(john anna dan camille)
    Drawy::Engine.couples = { 'john' => 'anna' }

    Drawy::Engine.draw(shuffle: false)

    assert_equal({ 'john' => 'dan', 'dan' => 'anna', 'anna' => 'camille', 'camille' => 'john' }, Drawy::Engine.result)
  end

  def test_build_list
    scenarii.each do |people, couples, expectation|
      Drawy::Engine.people  = people
      Drawy::Engine.couples = couples

      # Go to build_list method definition for a more detailed explanation
      # of where those expectations are coming from
      assert_equal expectation, Drawy::Engine.build_list
    end
  end

  def test_is_couple?
    Drawy::Engine.couples = { 'john' => 'anna' }

    assert_equal true,  Drawy::Engine.is_couple?('john', 'anna')
    assert_equal true,  Drawy::Engine.is_couple?('anna', 'john')
    assert_equal false, Drawy::Engine.is_couple?('dan', 'camille')
    assert_equal false, Drawy::Engine.is_couple?('john', nil)
  end

  private

  # Different scenarii to test edge cases
  # Each case is of form [people, couples, expectation]

  def scenarii
    [
      # couple at the beginning
      [%w(john anna dan camille), { 'john' => 'anna' }, %w(john dan anna camille)],
      # couple at the end
      [%w(john anna dan camille), { 'dan' => 'camille' }, %w(john camille anna dan)],
      # 2 couples only
      [%w(john anna dan camille), { 'john' => 'anna', 'dan' => 'camille' }, %w(john dan anna camille)],
      # 3 couples only
      [%w(john anna dan camille henrik luna), { 'john' => 'anna', 'dan' => 'camille', 'henrik' => 'luna' }, %w(john luna dan anna camille henrik)],
      # no couple
      [%w(john anna dan camille henrik luna), {}, %w(john anna dan camille henrik luna)]
    ]
  end
end