require 'test_helper'

class TestValidator < Minitest::Test
  def setup
    Drawy::Engine.people = nil
  end

  def test_validate_uniqueness_of_unique_names
    assert_nil Drawy::Validator.validate_uniqueness_of
    assert_nil Drawy::Validator.validate_uniqueness_of('john', 'anna')

    Drawy::Engine.people = %w(dan camille henrik)

    assert_nil Drawy::Validator.validate_uniqueness_of('john', 'anna')
  end

  def test_validate_uniqueness_of_used_names
    Drawy::Engine.people = ['john']

    assert_raises Drawy::ValidationError do
      Drawy::Validator.validate_uniqueness_of('john', 'anna')
    end
  end

  def test_validate_people_format
    [nil, [], ['john']].each do |people_value|
      Drawy::Engine.people = people_value

      assert_raises Drawy::ValidationError do
        Drawy::Validator.validate_people_format
      end
    end

    Drawy::Engine.people  = %w(john anna)
    Drawy::Engine.couples = { 'john' => 'anna' }

    assert_raises Drawy::ValidationError do
      Drawy::Validator.validate_people_format
    end

    Drawy::Engine.people  += ['dan']

    assert_raises Drawy::ValidationError do
      Drawy::Validator.validate_people_format
    end
  end

  def test_couple_plus_one?
    people  = %w(john anna dan)
    couples = { 'john' => 'anna' }

    assert_equal true, Drawy::Validator.couple_plus_one?(people, couples)

    people  = %w(john anna dan camille)
    couples = { 'john' => 'anna' }

    assert_equal false, Drawy::Validator.couple_plus_one?(people, couples)
  end
end